@@ -1,7 +1,7 @@
 use sqlparser::{
     ast::{
         Expr, Function, FunctionArg, ListAgg, ListAggOnOverflow, ObjectName, Query, Select,
-        SelectItem, SetExpr, Statement, TableFactor, TableWithJoins, WindowSpec, With,
+        SelectItem, SetExpr, TableFactor, TableWithJoins, WindowSpec, With,
     },
     dialect::{self, Dialect},
     tokenizer::{Token, Tokenizer},
@@ -39,23 +39,22 @@ impl SqlDialect {
 }
 
 pub struct Parser {
-    statement: Statement,
+    query: Query,
 }
 
 impl Parser {
-    pub fn new(sql_dialect: SqlDialect, query: &str) -> Self {
+    pub fn try_new(sql_dialect: SqlDialect, query: &str) -> Result<Self, &str> {
         let tokens = sql_dialect.tokenize(query);
-        let statement = sqlparser::parser::Parser::new(tokens, &*sql_dialect.sqlparser_dialect())
-            .parse_statement()
-            .unwrap();
-        Self { statement }
+        let query =
+            sqlparser::parser::Parser::new(tokens, &*sql_dialect.sqlparser_dialect()).parse_query();
+        match query {
+            Ok(query) => Ok(Self { query }),
+            Err(_) => Err("Failed to parse query"),
+        }
     }
 
     pub fn tables(&self) -> Vec<&str> {
-        match &self.statement {
-            Statement::Query(query) => from_query(query),
-            _ => todo!(),
-        }
+        from_query(&self.query)
     }
 }
 
@@ -337,7 +336,9 @@ mod tests {
     fn simple_select() {
         let query = "select * from cool_table";
         assert_eq!(
-            Parser::new(SqlDialect::PostgreSql, query).tables(),
+            Parser::try_new(SqlDialect::PostgreSql, query)
+                .unwrap()
+                .tables(),
             vec!["cool_table"]
         );
     }
@@ -345,7 +346,7 @@ mod tests {
     #[test]
     fn simple_union() {
         let query = "select * from cool_table union all select * from other_table";
-        let parser = Parser::new(SqlDialect::PostgreSql, query);
+        let parser = Parser::try_new(SqlDialect::PostgreSql, query).unwrap();
         let tables = parser.tables();
         assert_eq!(tables, vec!["cool_table", "other_table"]);
     }
@@ -359,7 +360,7 @@ mod tests {
             from table_a as a
             inner join table_b as b using (id_col)
         ";
-        let parser = Parser::new(SqlDialect::PostgreSql, query);
+        let parser = Parser::try_new(SqlDialect::PostgreSql, query).unwrap();
         let tables = parser.tables();
         assert_eq!(tables, vec!["table_a", "table_b"]);
     }
@@ -372,7 +373,7 @@ mod tests {
             )
             select * from my_cte
         ";
-        let parser = Parser::new(SqlDialect::PostgreSql, query);
+        let parser = Parser::try_new(SqlDialect::PostgreSql, query).unwrap();
         let tables = parser.tables();
         assert_eq!(tables, vec!["cool_table"])
     }
@@ -389,7 +390,7 @@ mod tests {
                 select * from actual_table
             ) as sub
         ";
-        let parser = Parser::new(SqlDialect::PostgreSql, query);
+        let parser = Parser::try_new(SqlDialect::PostgreSql, query).unwrap();
         let tables = parser.tables();
         assert_eq!(tables, vec!["actual_table", "unique_table"]);
     }
@@ -401,7 +402,7 @@ mod tests {
             union all
             select * from cool_table
         ";
-        let parser = Parser::new(SqlDialect::PostgreSql, query);
+        let parser = Parser::try_new(SqlDialect::PostgreSql, query).unwrap();
         let tables = parser.tables();
         assert_eq!(tables, vec!["cool_table"])
     }
@@ -422,7 +423,7 @@ mod tests {
                 where active_customers.customer_id = customers.customer_id
             )
         ";
-        let parser = Parser::new(SqlDialect::PostgreSql, query);
+        let parser = Parser::try_new(SqlDialect::PostgreSql, query).unwrap();
         let tables = parser.tables();
         assert_eq!(tables, vec!["active_customers", "customers", "sales"]);
     }
@@ -462,7 +463,7 @@ mod tests {
                 where my_lookup_table.my_lookup_id = sub.my_lookup_id
             )
         ";
-        let parser = Parser::new(SqlDialect::PostgreSql, query);
+        let parser = Parser::try_new(SqlDialect::PostgreSql, query).unwrap();
         let tables = parser.tables();
         assert_eq!(
             tables,
@@ -476,4 +477,11 @@ mod tests {
             ]
         );
     }
+
+    #[test]
+    fn invalid_query() {
+        let query = "selext column_a, column_b from my_table";
+        let parser = Parser::try_new(SqlDialect::PostgreSql, query);
+        assert!(parser.is_err());
+    }
 }
